@@ -19,11 +19,15 @@ class _WiserOpenThermBoilerParameters(object):
 
     @property
     def ch_setpoint_lower_bound(self) -> bool:
-        return self._data.get("maxChSetpointLowerBound", None)
+        return tf._from_wiser_temp(
+            self._data.get("maxChSetpointLowerBound", None), "current"
+        )
 
     @property
     def ch_setpoint_upper_bound(self) -> bool:
-        return self._data.get("maxChSetpointUpperBound", None)
+        return tf._from_wiser_temp(
+            self._data.get("maxChSetpointUpperBound", None), "current"
+        )
 
     @property
     def hw_setpoint_read_write(self) -> bool:
@@ -35,11 +39,19 @@ class _WiserOpenThermBoilerParameters(object):
 
     @property
     def hw_setpoint_lower_bound(self) -> bool:
-        return self._data.get("dhwSetpointLowerBound", None)
+        return tf._from_wiser_temp(
+            self._data.get("dhwSetpointLowerBound", None), "current"
+        )
 
     @property
     def hw_setpoint_upper_bound(self) -> bool:
-        return self._data.get("dhwSetpointUpperBound", None)
+        return tf._from_wiser_temp(
+            self._data.get("dhwSetpointUpperBound", None), "current"
+        )
+
+    @property
+    def json_data(self) -> dict:
+        return self._data
 
 
 class _WiserOpenThermOperationalData(object):
@@ -82,6 +94,10 @@ class _WiserOpenThermOperationalData(object):
         """Get SlaveStatus"""
         return self._data.get("SlaveStatus", None)
 
+    @property
+    def json_data(self) -> dict:
+        return self._data
+
 
 class _WiserOpentherm(object):
     """Data structure for Opentherm data"""
@@ -180,6 +196,10 @@ class _WiserOpentherm(object):
         """Get TrackedRoomId"""
         return self._data.get("TrackedRoomId", None)
 
+    @property
+    def json_data(self) -> dict:
+        return self._data
+
     async def set_opentherm_parameter(self, endpoint: str, cmd_data: str) -> bool:
         """Allow settign of opentherm param"""
         return await self._wiser_rest_controller._do_hub_action(
